@@ -2,13 +2,14 @@ import datetime
 import secrets
 from collections import defaultdict
 from operator import attrgetter
+from typing import Optional, List
 from uuid import uuid4
 
 import pypitoken
 import readme_renderer.markdown
 import readme_renderer.rst
 import readme_renderer.txt
-from flask import Flask, render_template, redirect, url_for, abort, request, g, flash
+from flask import Flask, render_template, redirect, url_for, abort, request, flash
 from flask_login import LoginManager, login_required, current_user, logout_user
 from flaskext.markdown import Markdown
 
@@ -33,15 +34,21 @@ def create_app(
     db: DBBackend,
     storage: PackageStorage,
     auth: Authenticator,
-    config: dict = None,
-    allow_project_creation=False,
+    session_secret: str = secrets.token_hex(16),
+    app_config: dict = None,
+    restrict_project_creation: Optional[List[str]] = None,
+    simple_api_allow_project_creation=False,
+    **kwargs,
 ):
     app = Flask(__name__)
-    if config:
-        app.config.update(**config)
+    app.secret_key = session_secret
+    if app_config:
+        app.config.update(**app_config)
     Markdown(app, extensions=["footnotes", "fenced_code"])
 
     log = app.logger
+    if kwargs:
+        log.warning(f"Unused options passed {list(kwargs.keys())}")
 
     # Setup Login and authentication
     login_manager = LoginManager(app)
@@ -57,13 +64,19 @@ def create_app(
     auth.init_app(app)
 
     simple_blueprint = simple_api.create_blueprint(
-        db, storage, allow_project_creation=allow_project_creation
+        db, storage, allow_project_creation=simple_api_allow_project_creation
     )
     app.register_blueprint(simple_blueprint)
 
     def get_user_id():
         return current_user.account.name
 
+    def check_project_creation_allowed():
+        return (
+            restrict_project_creation is None
+            or get_user_id() in restrict_project_creation
+        )
+
     @app.route("/logout")
     @login_required
     def logout():
@@ -140,8 +153,11 @@ def create_app(
             project for project in db.project_list() if project.visible(get_user_id())
         ]
         own_projects.sort(key=attrgetter("name"))
-
-        return render_template("project/projects.html", projects=own_projects)
+        return render_template(
+            "project/projects.html",
+            projects=own_projects,
+            show_create=check_project_creation_allowed(),
+        )
 
     @app.get("/projects/<project_name>")
     @login_required
@@ -190,6 +206,10 @@ def create_app(
     @app.route("/create_project", methods=["GET", "POST"])
     @login_required
     def create_project():
+        if not check_project_creation_allowed():
+            flash("Project creation restricted to specific users.")
+            return redirect(url_for("list_projects"))
+
         form = CreateProjectForm()
         if form.validate_on_submit():
             project = db.project_save(
@@ -236,27 +256,44 @@ def create_app(
 
         return render_template("project/project_users.html", project=project)
 
-    @app.post("/projects/<project_name>/admins")
+    @app.post("/projects/<project_name>/users")
     @login_required
-    def project_add_admin(project_name):
+    def project_users_add(project_name):
         project = db.project_get(project_name)
         if not project.is_admin(get_user_id()):
-            abort(401, 'You are not an admin, what are you doing here?')
+            abort(401, "You are not an admin, what are you doing here?")
 
         new_user = request.form.get("username")
-        if new_user:
-            log.info(f"{project_name} {get_user_id()} added {new_user} as admin")
-            project.admins.append(new_user)
-            db.project_save(project)
+        new_role = request.form.get("role")
+
+        if new_user and new_role:
+            # TODO check that user does not exist
+            if new_user in project.admins:
+                project.admins.remove(new_user)
+            if new_user in project.members:
+                project.members.remove(new_user)
+
+            log.info(f"{project_name} {get_user_id()} added {new_user} as {new_role}")
+            if new_role == "admin":
+                project.admins.append(new_user)
+            elif new_role == "member":
+                project.members.append(new_user)
+            else:
+                flash("Invalid role chosen!")
+
+            if len(project.admins) > 0:
+                db.project_save(project)
+            else:
+                flash("A project requires at least one admin.")
 
         return redirect(url_for("project_users", project_name=project_name))
 
-    @app.get("/projects/<project_name>/admins/<username>/delete")
+    @app.get("/projects/<project_name>/users/<username>/delete")
     @login_required
-    def project_remove_admin(project_name, username):
+    def project_users_remove(project_name, username):
         project = db.project_get(project_name)
         if not project.is_admin(get_user_id()):
-            abort(401, 'You are not an admin, what are you doing here?')
+            abort(401, "You are not an admin, what are you doing here?")
 
         if username in project.admins:
             # Never delete the last admin!
@@ -266,29 +303,6 @@ def create_app(
             else:
                 flash("A project requires at least one admin.")
 
-        return redirect(url_for("project_users", project_name=project_name))
-
-    @app.post("/projects/<project_name>/members")
-    @login_required
-    def project_add_member(project_name):
-        project = db.project_get(project_name)
-        if not project.is_admin(get_user_id()):
-            abort(401, 'You are not an admin, what are you doing here?')
-
-        new_user = request.form.get("username")
-        if new_user:
-            log.info(f"{project_name} {get_user_id()} added {new_user} as member")
-            project.members.append(request.form["username"])
-            db.project_save(project)
-        return redirect(url_for("project_users", project_name=project_name))
-
-    @app.get("/projects/<project_name>/members/<username>/delete")
-    @login_required
-    def project_remove_member(project_name, username):
-        project = db.project_get(project_name)
-        if not project.is_admin(get_user_id()):
-            abort(401, 'You are not an admin, what are you doing here?')
-
         if username in project.members:
             project.members.remove(username)
             db.project_save(project)
